/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (c) 2007 INRIA.  All Rights Reserved.
 * Author: Matthieu Fertré
 */

#include <linux/blkdev.h>
#include <linux/types.h>
#include <linux/kdb.h>
#include <linux/kdbprivate.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <asm/signal.h>
#include <kerrighed/krgnodemask.h>
#include <kerrighed/ctnr_headers.h>

MODULE_AUTHOR("IRISA - Matthieu Fertré");
MODULE_DESCRIPTION("Debug kddm objentry information");
MODULE_LICENSE("GPL");

#define NODE_IN_SET(set,nodeid) is_in_vector(set, nodeid)

static int
kdbm_kddm_obj(int argc, const char **argv)
{
	int diag;
	long ns_id, set_id, obj_id;

	if (argc != 3)
		return KDB_ARGCOUNT;

	diag = kdbgetularg((char *)argv[1], &ns_id);
	if (diag)
		return diag;
	diag = kdbgetularg((char *)argv[2], &set_id);
	if (diag)
		return diag;
	diag = kdbgetularg((char *)argv[3], &obj_id);
	if (diag)
		return diag;

	kdb_printf ("Namespace: %ld\n", ns_id);
	kdb_printf ("Set: %ld\n", set_id);
	kdb_printf ("Object: %ld\n", obj_id);

	if (kh_kddm_display_obj)
		return kh_kddm_display_obj(ns_id, set_id, obj_id);

	return KDB_NOTFOUND;
}

static int __init kdbm_kddm_init(void)
{
	kdb_register("kddm_obj", kdbm_kddm_obj, "<nsid> <setid> <objid>", "Display kddm local objentry", 0);

	return 0;
}

static void __exit kdbm_kddm_exit(void)
{
	kdb_unregister("kddm_obj");
}

kdb_module_init(kdbm_kddm_init)
kdb_module_exit(kdbm_kddm_exit)
