/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (c) 2007 Kerlabs.
 */

#include <linux/kdb.h>
#include <linux/kdbprivate.h>
#include <linux/module.h>
#include <linux/kallsyms.h>
#include <kerrighed/ctnr_headers.h>

MODULE_AUTHOR("Renaud Lottiaux / Kerlabs");
MODULE_DESCRIPTION("Display in memory Kerrighed debug logs");

static int generic_kddm_log_display (int limit, int set_id, int obj_id)
{
	struct kddm_log *log;
	int i = 0, n = 0;
	char buffer[256];

	if (!kh_kddm_print_log) {
		printk("Cannot display logs. Kerrighed hook not registered\n");
		return 0;
	}
	
	log = kddm_log_head;
	
	while (log) {
		i++;
		if (i < limit)
			goto next;
		if ((set_id != 0) && (set_id != log->set_id))
			goto next;
		if ((obj_id != -1UL) && (obj_id != log->obj_id))
			goto next;
		
		kh_kddm_print_log(log, buffer);
		kdb_printf ("%s", buffer);
		n++;
next:
		log = log->next;
	};
	printk ("%d/%d logs displayed\n", n, atomic_read (&nr_kddm_logs));
	
	return 0;
}

static int kdbm_log(int argc, const char **argv)
{
	unsigned long nr;
	int err, limit = 0;

	if (argc > 1)
		return KDB_ARGCOUNT;

	if (argc == 1) {
		err = kdbgetularg((char *)argv[1], &nr);
		if (err)
			return err;
		limit = atomic_read (&nr_kddm_logs) - nr;
	}

	return generic_kddm_log_display (limit, 0, -1UL);
}

static int kdbm_log_filter(int argc, const char **argv)
{
	int err, limit = 0;
	unsigned long nr, set_id = 0, obj_id = -1UL;

	if ((argc < 2) || (argc > 3))
		return KDB_ARGCOUNT;

	err = kdbgetularg((char *)argv[1], &set_id);
	if (err)
		return err;
	err = kdbgetularg((char *)argv[2], &obj_id);
	if (err)
		return err;

	if (argc == 3) {
		err = kdbgetularg((char *)argv[3], &nr);
		if (err)
			return err;
		limit = atomic_read (&nr_kddm_logs) - nr;
	}

	return generic_kddm_log_display(limit, set_id, obj_id);
}

static int __init kdbm_log_init(void)
{
	kdb_register ("kddm_log", kdbm_log, "[lines]",
		      "Display KDDM logs", 0);

	kdb_register ("kddm_log_filter", kdbm_log_filter,
		      "<sid> <oid> [lines]",
		      "Display KDDM logs", 0);

	return 0;
}

static void __exit kdbm_log_exit(void)
{
	kdb_unregister("log");
}
kdb_module_init(kdbm_log_init)
kdb_module_exit(kdbm_log_exit)
